@@ -1,6 +1,8 @@
-#' For a k=1 Gegenbauer process, use semi-parametric methods to estimate the Gegenbauer frequency and fractional differencing.
+#' Extract semiparametric estimates of the Gegenbauer factors.
+#'
+#' For a Gegenbauer process, use semi-parametric methods to estimate the Gegenbauer frequency and fractional differencing.
 #' @param x (num) This should be a numeric vector representing the process to estimate.
-#' @param k (int) The number of Gegenabuer frequencies
+#' @param k (int) The number of Gegenbauer frequencies
 #' @param alpha (num)
 #' @param method (char) One of "gsp" or "lpr" - lpr is the log-periodogram-regression technique, "gsp" is the Gaussian
 #' semi-parametric technique. "gsp" is the default. Refer Arteche (1998).
@@ -15,7 +17,8 @@
 #' @export
 ggbr_semipara <- function(x,k=1,alpha=0.8,method='gsp',min_freq=0.0,max_freq=0.5) {
   if (!method%in%c('gsp','lpr')) stop('Invalid method. Should be one of "gsp" or "lpr".')
-  if (alpha<=0|alpha>=1) stop('alpha should be between 0 and 1, but not0 and not 1.')
+  if (alpha<=0|alpha>=1) stop('alpha should be between 0 and 1, but not 0 and not 1.')
+  x <- as.numeric(x)
   k <- as.integer(k)
   if (k<=0) stop('k should be a small positive integer.')
   peak_idx_list <- c()
@@ -58,17 +61,21 @@ ggbr_semipara <- function(x,k=1,alpha=0.8,method='gsp',min_freq=0.0,max_freq=0.5
 print.ggbr_factors<-function(x,...) {
   printf_9_4<-function(f) cat(sprintf('%9.4f',f))
 
-  cat('                      ')
-  for (k1 in 1:length(x)) cat(sprintf('%9.9s',paste0('Factor',k1)))
-  cat('\nGegenbauer frequency: ')
-  for (factor in x) printf_9_4(factor$f)
-  cat('\nGegenbauer Period:    ')
-  for (factor in x) printf_9_4(1/factor$f)
-  cat('\nGegenbauer Exponent:  ')
-  for (factor in x) printf_9_4(factor$fd)
-  cat('\n')
+  if (length(x)>0) {
+    cat('                      ')
+    for (k1 in 1:length(x)) cat(sprintf('%9.9s',paste0('Factor',k1)))
+    cat('\nGegenbauer frequency: ')
+    for (factor in x) printf_9_4(factor$f)
+    cat('\nGegenbauer Period:    ')
+    for (factor in x) printf_9_4(1/factor$f)
+    cat('\nGegenbauer Exponent:  ')
+    for (factor in x) printf_9_4(factor$fd)
+    cat('\n')
+  } else cat('No Gegenbauer factors.\n')
 }
 
+#' Print Semiparametric Estimates
+#'
 #' Print a semiparameteric Gegenbauer estimation object.
 #' @param x An object of class garma_semipara.
 #' @param ... further parameters for print function
@@ -81,9 +88,11 @@ print.garma_semipara<-function(x,...) {
   if(length(x$note)>0) if (nchar(x$note)>1) cat(paste0('\n',x$note))
 }
 
-#' For a k=1 Gegenbauer process, transform to remove Gegenbauer long memory component to get a short memory (ARMA) process.
-#' @param x (num) This should be a numeric vector representing the process to estimate.
-#' @param ggbr_factors (list) Each element of the list represents a Gegenbauer factor and includes f, u and fd elements.
+#' Extract underlying ARMA process.
+#'
+#' For a Gegenbauer process, transform to remove Gegenbauer long memory component to get a short memory (ARMA) process.
+#' @param x (num) This should be a numeric vector representing the Gegenbauer process.
+#' @param ggbr_factors (class) Each element of the list represents a Gegenbauer factor and includes f, u and fd elements.
 #' @return An object of same class as x.
 #' @examples
 #' data(AirPassengers)
@@ -107,10 +116,12 @@ extract_arma<-function(x,ggbr_factors) {
 }
 
 .garma_pgram<-function(x) {
+  x <- as.numeric(x) # If we don't do this, the spectrum function returns 'different' frequencies.
   return(spectrum(x,plot=F,detrend=FALSE,demean=FALSE,method='pgram',taper=0,fast=FALSE))
 }
 
-.yajima_ggbr_freq<-function(x,remove_peaks,min_freq=0.0,max_freq=0.5) {
+.yajima_ggbr_freq<-function(x,remove_peaks,min_freq=0.0,max_freq=2*pi) {
+  x <- as.numeric(x) # If we don't do this, the spectrum function returns 'different' frequencies.
   ssx       <- .garma_pgram(x)
 
   if (!is.null(min_freq)&!is.null(max_freq)) {
@@ -139,37 +150,37 @@ extract_arma<-function(x,ggbr_factors) {
   return(list(f_idx=f_idx, ggbr_freq=ggbr_freq, ssx=ssx))
 }
 
-.gsp<-function(x,alpha,remove_peaks,min_freq=0.0,max_freq=1.0) {
+.gsp<-function(x,alpha,remove_peaks,min_freq=0.0,max_freq=2*pi) {
   # as per Arteche 1998 "SEMIPARAMETRIC INFERENCE IN SEASONAL AND CYCLICAL LONG MEMORY PROCESSES"
   # determine "fd"
-  c_fcn<-function(fd, omega, spec) {return(mean((omega^(2*fd)) * spec,rm.na=TRUE))}
-  r_fcn<-function(fd, f_idx, ssx) {
-    omega <- 2*pi*ssx$freq[1:(m-1)]
-    spec1 <- ssx$spec[(f_idx+2):(f_idx+m)]
-    min_idx <- f_idx - m
-    if (m<f_idx+1) spec2 <- ssx$spec[(f_idx-m):(f_idx-2)]
-    else {
-      if (f_idx>2) spec2 <- c(ssx$spec[(f_idx-2):1], ssx$spec[length(ssx$spec):(length(ssx$spec)-(m-f_idx))])
-      else spec2 <- c(ssx$spec[length(ssx$spec):(length(ssx$spec)-(m-f_idx))])
-      spec2 <- spec2[1:(m-1)]
-    }
+  c_fcn<-function(fd, omega, spec) {return(mean((omega^(2*fd)) * spec,na.rm=TRUE))}
+  r1_fcn<-function(fd, f_idx, ssx) {
+    omega <- 2*pi*ssx$freq[1:m]  # Frequencies
 
-    res <- log(c_fcn(fd, omega, spec1)) + log(c_fcn(fd, omega, spec2)) - 4*fd*mean(log(omega),rm.na=TRUE)
+    # Spec to use, as offset from ggbr_freq. These are specs above ggbr_freq.
+    spec_2pi <- c(ssx$spec,rev(ssx$spec))
+    spec1 <- spec_2pi[f_idx:(f_idx+m)]
+    spec1 <- spec1[1:m]
+
+    res <- log(c_fcn(fd, omega, spec1)) - 2*fd*mean(log(omega),na.rm=TRUE)
     if (is.infinite(res)|is.na(res)|is.null(res)) res<-1e200
     return(res)
   }
+
+  x <- as.numeric(x) # If we don't do this, the spectrum function returns 'different' frequencies.
   # first identify the peak - the Gegenbauer frequency
   yf <- .yajima_ggbr_freq(x,remove_peaks,min_freq,max_freq)
   m  <- as.integer((length(x)/2)^alpha)
 
-  fd <- stats::optimise(r_fcn, f_idx=yf$f_idx, ssx=yf$ssx, lower=-10, upper=10)$minimum / 2
+  fd <- stats::optimise(r1_fcn, f_idx=yf$f_idx, ssx=yf$ssx, lower=-10, upper=10)$minimum
   u  <- cos(2*pi*yf$ggbr_freq)
 
   return(list(fd=fd,f=yf$ggbr_freq,u=u,m=m,f_idx=yf$f_idx))
 }
 
-.lpr<-function(x,alpha,remove_peaks,min_freq=0.0,max_freq=1.0) {
+.lpr<-function(x,alpha,remove_peaks,min_freq=0.0,max_freq=2*pi) {
   # first identify the peak - the Gegenbauer frequency
+  x <- as.numeric(x) # If we don't do this, the spectrum function returns 'different' frequencies.
   yf       <- .yajima_ggbr_freq(x,remove_peaks,min_freq,max_freq)
   ssx      <- yf$ssx
   f_idx    <- yf$f_idx
